<?php
// $Id: pushkey.drush.inc,v 1.1 2010/10/19 22:35:46 greg1anderson Exp $

/**
 * @file
 * pushkey - Drush commands to create a public/private key pair
 * and push the public key to a remote account.
 *
 * Copyright (C) 2010  Greg Anderson <http://drupal.org/user/438598>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

function pushkey_drush_help($section) {
  switch ($section) {
    case 'drush:pushkey':
      return dt('Generate a public/private key pair and push to a remote machine.');
  }
}

function pushkey_drush_command() {
  $items = array();

  $items['pushkey'] = array(
    'description' => 'Generate a public/private key pair and push the public key to a remote server.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap.
    'options' => array(),
    'arguments' => array(
      'target' => 'The user and machine to push to ("user@machine.com").',
    ),
    'examples' => array(
      'drush pushkey user@myserver.com' => 'Push your public key to a remote server.',
    ),
  );

  return $items;
}

function drush_pushkey($target = NULL) {
  // Make sure that we were called with a parameter
  if ($target == NULL) {
    return drush_set_error('DRUSH_NO_TARGET', dt('No target was specified.'));
  }

  if (is_null(drush_server_home())) {
    return drush_set_error('DRUSH_NO_HOME', dt('Cannot determine appropriate home directory.'));
  }
  
  $local_machine_name = exec('uname -n');
  
  $remote_machine = $target;
  $at_pos = strpos($target, '@');
  if ($at_pos !== FALSE) {
    $remote_user = substr($target, 0, $at_pos);
    $remote_machine = substr($target, $at_pos + 1);
  }
  
  // If the user was not specified with the target,
  // then assume that the remote user == local user
  if (!isset($remote_user)) {
    $remote_user = exec('whoami');
  }
  
  // Figure out where the remote home directory is
  $remote_home = "/home/$remote_user";
  if ($remote_user == 'root') {
    $remote_home = "/root";
  }
  
  $remote_machine_with_user = "$remote_user@$remote_machine";
  
  // Make sure that the .ssh directory exists
  $ssh_dir = drush_server_home() . '/.ssh';
  if (!is_dir($ssh_dir)) {
    mkdir($ssh_dir);
  }

  // If we do not have an id_rsa file, then make one
  $private_key_file = $ssh_dir . '/id_rsa';
  $public_key_file = $ssh_dir . '/id_rsa.pub';
  
  if (!is_file($private_key_file)) {
    drush_op('system', 'ssh-keygen -t rsa -f ' . $private_key_file);
  }
  
  // Read in the public key data
  if (is_file($public_key_file)) {
    $public_key_data = trim(file_get_contents($public_key_file));
  }
  elseif (drush_get_context('DRUSH_SIMULATE')) {
    $public_key_data = "SIMULATED_PUBLIC_KEY_DATA";
  }
  
  // Object if we still do not have a public key file
  if (!isset($public_key_data)) {
    return drush_set_error('DRUSH_NO_KEY', dt('Could not find or create a public / private key pair.'));
  }

  // Read in the public key data
  $remote_public_key_file = $local_machine_name . '_id_rsa.pub';
  
  // Build the remote command and then run it
  $exec = "ssh $remote_machine_with_user /bin/sh -c \"'mkdir -p $remote_home/.ssh && (echo $public_key_data > $remote_home/.ssh/$remote_public_key_file) && (cat $remote_home/.ssh/$remote_public_key_file >> $remote_home/.ssh/authorized_keys) && chmod 600 $remote_home/.ssh/authorized_keys'\"";
  $result = drush_op('system', $exec); 
}
